@@ -27,19 +27,40 @@ class ds_query(object):
 class vquery(ds_query):
     def __init__(self, vview_gen):
 	self.vview_gen = vview_gen
+        self.argcount = vview_gen.func_code.co_argcount
 	pass
     
     def __get__(self, instance, owner):
+        try:
+            return instance.__cmds[self]
+        except:
+            pass
+
 	def cmd(*args):
+            assert len(args) == self.argcount
+
 	    if hasattr(self, 'qstr'):
 		qstr = self.qstr
+                args_temp = self.args_temp
 	    else:
-		qstr = str(self.vview_gen())
-		self.qstr = qstr
+                free_vars = [sql.free_var_temp(i)
+                             for i in range(self.argcount)]
+                vv = self.vview_gen(*free_vars)
+		self.qstr = qstr = str(vv)
+                self.args_temp = args_temp = vv.get_const_values()
 		pass
+            exec_args = [((isinstance(v, sql.free_var) and (args[v.idx],)) or
+                          (v,))[0]
+                         for v in args_temp]
 	    cu = instance.get_cursor()
-	    cu.execute(qstr, args)
+	    cu.execute(qstr, exec_args)
 	    return cu.fetchone()[0]
+
+        try:
+            instance.__cmds[self] = cmd
+        except AttributeError:
+            instance.__cmds = {self: cmd}
+            pass
 	return cmd
     pass
 
@@ -48,20 +69,42 @@ class squery(ds_query):
     def __init__(self, field_names, vview_gen):
 	self.field_names = field_names
 	self.vview_gen = vview_gen
+        self.argcount = vview_gen.func_code.co_argcount
 	pass
     
     def __get__(self, instance, owner):
+        try:
+            return instance.__cmds[self]
+        except:
+            pass
+
 	def cmd(*args):
+            assert len(args) == self.argcount
+
 	    from pythk.conveniences import combine_sequences
 	    if hasattr(self, 'qstr'):
 		qstr = self.qstr
+                args_temp = self.args_temp
 	    else:
-		self.qstr = qstr = str(self.vview_gen())
+                free_vars = [sql.free_var_temp(i)
+                             for i in range(self.argcount)]
+                vv = self.vview_gen(*free_vars)
+		self.qstr = qstr = str(vv)
+                self.args_temp = args_temp = vv.get_const_values()
 		pass
+            exec_args = [((isinstance(v, sql.free_var) and (args[v.idx],)) or
+                          (v,))[0]
+                         for v in args_temp]
 	    cu = instance.get_cursor()
-	    cu.execute(qstr, args)
+	    cu.execute(qstr, exec_args)
 	    row = cu.fetchone()
 	    return dict(combine_sequences(self.field_names, row))
+
+        try:
+            instance.__cmds[self] = cmd
+        except AttributeError:
+            instance.__cmds = {self: cmd}
+            pass
 	return cmd
     pass
 
@@ -75,20 +118,42 @@ class mquery(ds_query):
     def __init__(self, field_names, vview_gen):
 	self.field_names = field_names
 	self.vview_gen = vview_gen
+        self.argcount = vview_gen.func_code.co_argcount
 	pass
     
     def __get__(self, instance, owner):
+        try:
+            return instance.__cmds[self]
+        except:
+            pass
+
 	def cmd(*args):
+            assert len(args) == self.argcount
+
 	    from pythk.conveniences import combine_sequences
 	    if hasattr(self, 'qstr'):
 		qstr = self.qstr
+                args_temp = self.args_temp
 	    else:
-		self.qstr = qstr = str(self.vview_gen())
+                free_vars = [sql.free_var_temp(i)
+                             for i in range(self.argcount)]
+                vv = self.vview_gen(*free_vars)
+		self.qstr = qstr = str(vv)
+                self.args_temp = args_temp = vv.get_const_values()
 		pass
+            exec_args = [((isinstance(v, sql.free_var) and (args[v.idx],)) or
+                          (v,))[0]
+                         for v in args_temp]
 	    cu = instance.get_cursor()
-	    cu.execute(qstr, args)
+	    cu.execute(qstr, exec_args)
 	    rows = cu.fetchall()
 	    return [dict(combine_sequences(self.field_names, row)) for row in rows]
+
+        try:
+            instance.__cmds[self] = cmd
+        except AttributeError:
+            instance.__cmds = {self: cmd}
+            pass
 	return cmd
     pass
 
@@ -243,16 +308,23 @@ if __name__ == '__main__':
                 pass
     
             @mquery_fs('value', 'date')
-            def user_trans():
+            def user_trans(uname):
                 u = users()
                 t = transactions()
                 return (u * t).fields(t.value, t.date).\
-                    where((u.sn == t.user) & (u.name == _q))
+                    where((u.sn == t.user) & (u.name == uname))
+    
+            @mquery_fs('value', 'date')
+            def snuser_trans(sn, uname):
+                u = users()
+                t = transactions()
+                return (u * t).fields(t.value, t.date).\
+                    where((u.name == uname) & (u.sn == sn))
     
             @vquery
-            def user_balance():
+            def user_balance(uname):
                 u = users()
-                return vview(u).fields(u.balance).where(u.name == _q)
+                return vview(u).fields(u.balance).where(u.name == uname)
 
             # ds_define() collects the definition of the data model.
             return ds_define()
@@ -291,6 +363,7 @@ if __name__ == '__main__':
     print ds.transactions.schema
     ds.init_db()
     ds.user_trans('test')
+    ds.snuser_trans(1, 'test')
     ds.user_balance('test')
     ds.insert(ds.users, sn=1, name='user1', balance=0)
     ds.delete(ds.users, ds.users.sn == 333)
