@@ -1,6 +1,9 @@
 '''
 sql.py: Host SQL on Python Language
 
+This module define a language for create data virtual views.  These
+vviews are mapped to SELECT command of SQL.
+
 table * table ==> vview
 vview * table ==> vview
 
@@ -211,16 +214,10 @@ class const(operand):
     def __deepcopy__(self, memo):
 	return const(self.value)
     
-    @staticmethod
-    def str_escape(txt):
-	return '\'' + txt.replace('\'', '\'\'') + '\''
-    
     def get_sql_name(self):
 	import types
 	if self.value is not None:
-	    if isinstance(self.value, types.StringTypes):
-		return self.str_escape(self.value)
-	    return repr(self.value)
+            return '?'
 	return 'NULL'
     
     def get_sql_name_no_alias(self):
@@ -236,7 +233,7 @@ class const(operand):
 # ============================================================
 class sys_sym(operand):
     sym = None
-    def __depcopy__(self, memo):
+    def __deepcopy__(self, memo):
 	return self
     
     def get_sql_name(self):
@@ -266,6 +263,13 @@ class free_var(sys_sym):
     pass
 _q = free_var()
 
+class free_var_temp(free_var):
+    def __init__(self, idx):
+        super(free_var_temp, self).__init__()
+        self.idx = idx
+        pass
+    pass
+
 # =============================================================
 class SQL_field_type(operand):
     '''Define data type of SQL
@@ -342,6 +346,11 @@ class str_f(SQL_field_type):
     type_name = 'text'
     pass
 
+class primary_sn_f(SQL_field_type):
+    type_name = 'integer primary key autoincrement'
+    fmt_symbol = '%d'
+    pass
+
 # ============================================================
 # constraints
 class unique_c(object):
@@ -607,6 +616,13 @@ class vview(object):
 	self.__link_fields_to_new_tables(new_view, self.__members, new_view.__members)
 	
 	return new_view
+
+    def get_const_values(self):
+        values = ()
+        for c in self.__cond:
+            values = values + c.get_const_values()
+            pass
+        return values
     
     def make_query_str(self, restrict=None):
 	tbs = string.join(map(lambda x: x.tab_name + ' ' + x.get_alias(), self.__members), ', ')
@@ -728,15 +744,16 @@ if __name__ == '__main__':
     view.fields(t1.f1, foo(t1.f2)).where((t1.f1 == 'abc') & (foo(t1.f2) < t1.f3) & (t1.f3 > 4) & (t1.f2 == _q))
     print 'Virtual view =============================='
     print str(view)
+    print 'Arguments ================================='
+    print view.get_const_values()
     print 'Schema ===================================='
     print t1.schema
     print 'Deepcopy table ============================'
     print deepcopy(t1).schema
     print 'Deepcopy vview ============================'
     print str(deepcopy(view))
-    print str(view)
     print 'Gen_update_cmd ============================'
-    print t1.factory(f1=_q, f2='aa', f3=11.5).gen_update_cmd(t1.f2==_q)
+    print t1.factory(f1=_q, f2='aa', f3=11.5).gen_update_cmd(t1.f2=='ff')
     print 'Gen_insert_cmd ============================'
     print t1.factory(f1=1, f2='aa', f3=11.5).gen_insert_cmd()
     print 'Gen_delete_cmd ============================'
